@@ -15,15 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'chord',
-    'stock-icon': 'solfege-chord',
-    'menupath': '/Menubar/FileMenu/ChordsMenu',
-    'name': _('_Chords'),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gtk, gnome
 import gobject
 import gu, widgets
@@ -34,7 +25,6 @@ import soundcard
 import dataparser
 
 class Teacher(abstract.LessonbasedTeacher):
-    exercise_data = __exercise_data__
     OK = 0
     ERR_PICKY = 1
     # valid values for self.q_status:
@@ -44,14 +34,9 @@ class Teacher(abstract.LessonbasedTeacher):
     # QSTATUS_GIVE_UP  after 'Give Up' has been pressed.
     CORRECT = 1
     ALL_CORRECT = 2
-    def __init__(self, exname, app, lesson_collection, lesson_file, config):
-        abstract.LessonbasedTeacher.__init__(self, exname, app,
-              lesson_collection, lesson_file, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.ChordLessonfile
-        try: # try: because the file can have changed since startup
-            self.parse_lessonfile()
-        except dataparser.DataparserException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
     def new_question(self):
         """
         return OK or ERR_PICKY
@@ -191,7 +176,6 @@ class Gui(abstract.LessonbasedGui):
         # config_box #
         ##############
         self.config_box.set_spacing(gnome.ui.PAD_SMALL)
-        self.add_select_lessonfile_gui()
         self.add_random_transpose_gui()
         # -----------------------------------------
         self.g_select_questions_category_box, category_box= gu.hig_category_vbox(
@@ -202,16 +186,13 @@ class Gui(abstract.LessonbasedGui):
         category_box.pack_start(self.g_select_questions, False)
         self.g_select_questions.show()
 
-        self.g_instrument_configurator  \
-              = widgets.InstrumentConfigurator(self.m_exname, 3)
-        self.config_box.pack_start(self.g_instrument_configurator, False)
-        self.g_instrument_configurator.show()
         self.update_gui_after_lessonfile_change()
     def update_select_question_buttons(self):
-        if self.m_t.m_P and self.m_t.m_P.header.questions_selectable:
+        if self.get_bool('gui/expert_mode'):
             self.g_select_questions_category_box.show()
             self.g_select_questions.initialize(self.m_t.m_P.header.fillnum,
                                  self.m_t.m_P.header.filldir)
+            self.m_t.check_askfor()
             for question in self.m_t.m_P.iterate_questions_with_unique_names():
                 self.g_select_questions.add(question, 'normal')
         else:
@@ -259,6 +240,8 @@ class Gui(abstract.LessonbasedGui):
     def update_gui_after_lessonfile_change(self):
         self.g_music_displayer.clear()
         if self.m_t.m_P:
+            self.update_select_question_buttons()
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
             self.g_new.set_sensitive(True)
             self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
             if not [q for q in self.m_t.m_P.m_questions if 'toptone' in q]:
@@ -273,16 +256,12 @@ class Gui(abstract.LessonbasedGui):
             else:
                 self.g_inversion_label.show()
                 self.g_vsep1.show()
-            if self.m_t.m_P.header.questions_selectable:
-                self.m_t.check_askfor()
         else:
             self.g_new.set_sensitive(False)
         self.g_repeat.set_sensitive(False)
         self.g_repeat_arpeggio.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
         self.update_answer_buttons()
-        self.g_win.set_title("Solfege - " + self.get_pretty_name())
-        self.update_select_question_buttons()
     def on_type(self, button, event=None):
         if event is None:
             self.on_type_left_clicked(button)
