@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import gtk
-
-__exercise_data__ = {
-    'exercise_name': 'chord-voicing',
-    'menu_path': '%s/%s' % (_("_Chords"), _("_Chord voicing")),
-    'stock-icon': 'solfege-chord-voicing',
-    'toolbar': 1,
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
+import gobject
 
 import gnome
 
@@ -42,16 +34,10 @@ import soundcard
 class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
-    exercise_data = __exercise_data__
     #UGH should we do this here
-    def __init__(self, exname, app, lesson_collection, lesson_file, config):
-        abstract.LessonbasedTeacher.__init__(self, exname, app,
-            lesson_collection, lesson_file, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.ChordLessonfile
-        try: # try: because the file can have changed since startup
-            self.parse_lessonfile()
-        except dataparser.DataparserException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
     def new_question(self):
         """
         return 1 if the teacher has a question to ask
@@ -137,7 +123,7 @@ class Gui(abstract.LessonbasedGui):
         #################
         # stacking frame
         #################
-        self.g_stacking_frame = gtk.Frame(_("Click to stack in the correct order"))
+        self.g_stacking_frame = gtk.Frame(_("Chord voicing"))
         self.g_stacking_frame.set_sensitive(False)
         hbox.pack_start(self.g_stacking_frame)
         vbox = gtk.VBox()
@@ -167,7 +153,6 @@ class Gui(abstract.LessonbasedGui):
         # config_box #
         ##############
         self.config_box.set_spacing(gnome.ui.PAD_SMALL)
-        self.add_select_lessonfile_gui()
         self.add_random_transpose_gui()
         self.update_gui_after_lessonfile_change()
     def new_question(self, widget=None):
@@ -193,7 +178,7 @@ class Gui(abstract.LessonbasedGui):
                 for c in self.g_chordtype_box.get_children():
                     c.set_sensitive(True)
                 self.m_t.m_P.play_question()
-                self.g_flashbar.flash(_("Identify chord type"))
+                self.g_flashbar.flash(_("Chord type"))
                 self.g_chordtype_box.get_children()[0].grab_focus()
             elif g == Teacher.ERR_PICKY:
                 self.g_flashbar.flash(_("You have to solve this question first."))
@@ -208,6 +193,7 @@ class Gui(abstract.LessonbasedGui):
                self.m_t.m_P._idx, self.get_string('lessonfile'), e)
     def update_gui_after_lessonfile_change(self, *v):
         if self.m_t.m_P:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
             self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
             self.g_new.set_sensitive(True)
             self.fill_chordtype_box()
@@ -216,7 +202,6 @@ class Gui(abstract.LessonbasedGui):
         self.g_repeat.set_sensitive(False)
         self.g_repeat_arpeggio.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
-        self.g_win.set_title("Solfege - " + self.get_pretty_name())
         self.clear_stacking_frame()
     def clear_stacking_frame(self):
         for c in self.g_source.get_children() + self.g_answer.get_children():
@@ -293,7 +278,7 @@ class Gui(abstract.LessonbasedGui):
             else:
                 self.g_flashbar.flash(_("Wrong"))
         elif self.m_t.q_status in (const.QSTATUS_TYPE_SOLVED, const.QSTATUS_VOICING_WRONG):
-            self.g_flashbar.flash(_("Type is already solved, now specify voicing"))
+            self.g_flashbar.flash(_("Type is already solved, now specify voicing."))
     def on_notebutton_clicked(self, btn, n, user_notename):
         newb = gtk.Button(user_notename)
         newb.show()
@@ -345,8 +330,12 @@ class Gui(abstract.LessonbasedGui):
         for c in self.g_chordtype_box.get_children():
             c.set_sensitive(c.get_data('chordtype')==t)
     def on_start_practise(self):
+        self.g_flashbar.require_size([
+            _("You have to solve this question first."),
+            _("Type is already solved, now specify voicing."),
+        ])
         self.g_new.grab_focus()
-        gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
+        gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New chord' to begin.")))
         self.fill_chordtype_box()
         self.clear_stacking_frame()
