@@ -24,6 +24,7 @@ except:
     import i18n
     i18n.setup_srcdir()
 
+import urlparse
 import os
 import gtk
 import re, math
@@ -31,6 +32,7 @@ import mpd
 import const
 import random
 import soundcard, cfg
+import unittest
 
 def mangle_email(email):
     email = email.replace("@", " ,, ")
@@ -153,31 +155,17 @@ def un_escape_url_string(s):
         return chr(eval("0x%s" % m.groups()[1]))
     return r.sub(f, s)
 
-
-def parse_url(s):
-    # protocol, action, fn, collection, lessonfile, config
-    r = re.compile("""((?P<protocol>\w+):)? # protocol
-           ((?P<action>\w[\w-]*))? #practise|config|statistics))? # 
-           (/(?P<fn>[\w\-\.]+))? # exercise or html file name
-           ((?P<anchor>\#\w+))? # anchor
-           (/(?P<collection>\w+))? # collection
-           (/(?P<lessonfile>[\.\w-]+))? # lesson file
-           (\?(?P<config>.*))? """, re.VERBOSE|re.DOTALL)
-    m = r.match(s)
-    if not m.group('protocol'):
-        if '#' in s:
-            return [None, None] + s.split('#', 1) + [None, None, {}]
-        return None, None, s, None, None, None, {}
+def _str_to_dict(s):
     D = {}
-    if m.group('config'):
-        V = m.group('config').split(";")
+    if s:
+        V = s.split(";")
         for e in V:
             n, v = e.split("=")
             D[n.strip()] = v.strip()
     for k in D:
         D[k] = un_escape_url_string(D[k])
-    return m.group('protocol'), m.group('action'), m.group('fn'), \
-           m.group('anchor'), m.group('collection'), m.group('lessonfile'), D
+    return D
+
 
 def get_modifier(s):
     m = (('<ctrl>', gtk.gdk.CONTROL_MASK),
@@ -251,9 +239,52 @@ def compare_version_strings(A, B):
         return -1
     return 0
 
+class Url:
+    __test__ = ">>> True"
+    def __init__(self, href):
+        self.href = href
+        self.protocol, loc, path, params, query, self.anchor = urlparse.urlparse(self.href)
+        if not self.protocol:
+            self.filename = path
+        elif self.protocol == 'solfege':
+            v = path.split("/")
+            self.action = v[0]
+            if len(v) > 1:
+                self.lessoncollection = v[1]
+            else:
+                self.lessoncollection = None
+            if len(v) > 2:
+                self.lessonfile = v[2]
+            else:
+                self.lessonfile = None
+            if len(v) > 2 and self.lessonfile and '?' in self.lessonfile:
+                self.lessonfile, self.config = self.lessonfile.split('?', 1)
+            else:
+                self.config = {}
+            if self.config:
+                self.config = _str_to_dict(self.config)
+
+class TestUrlClass(unittest.TestCase):
+    def setUp(self):
+        pass
+    def test_all(self):
+        url = Url('solfege:practise/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
+        self.assert_(url.protocol == 'solfege')
+        self.assert_(url.action == 'practise')
+        self.assert_(url.lessoncollection == 'solfege')
+        self.assert_(url.lessonfile == 'melodic-intervals')
+        self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
+        url = Url('solfege:/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
+        self.assert_(url.protocol == 'solfege')
+        self.assert_(url.lessoncollection == 'solfege')
+        self.assert_(url.lessonfile == 'melodic-intervals')
+        self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
+        url = Url('solfege:/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
+        self.assert_(url.protocol == 'solfege')
+        self.assert_(url.lessoncollection == 'solfege')
+        self.assert_(url.lessonfile == 'melodic-intervals')
+        self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
+
+
 if __name__ == '__main__':
-    print parse_url("main.html#123")
-    print parse_url("main.html")
-    print parse_url("exercise:/main.html#23")
-    print parse_url("solfege:practise/id-by-name/solfege/fifth-small-pure-0.99")
-    print parse_url("solfege:/harmonic-interval?intervals=[1, 2, 3]")
+    unittest.main()
